% This template is used to create fourier analysis images, segmentation and
% SASTT loading data analysis
close all
clear variables
addpath ../
%% STEP 1: give the locations to find the raw data and to save the processed data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
par.online_analysis = 1;            %  = 1 for online analysis, =0 for offline, needs user_name
par.sample_name = 'sample_name';    %  name for the subfolders
user_name = '17041'; % e-account number, without 'e', used only if par.online_analysis = 0
fmt.parallel_plotting = false;      % = true will plot using a parfor, 
                                    % plots will not be shown on the screen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if par.online_analysis
    % use online (e-account)
    par.user_name = beamline.identify_eaccount;
    base_dir_beamtime = '~/Data10/';
    base_dir_work = base_dir_beamtime;
else % for off-line analysis: Daas
    par.user_name = ['e' user_name];
    % use offline (p-group)
    base_dir_beamtime = sprintf('/sls/X12SA/Data20/e%s/', user_name);
    base_dir_work = sprintf('/das/work/p%s/p%s/', user_name(1:2), user_name);
end
fnames.data_dir  = sprintf('%s/analysis/', base_dir_beamtime); % Where the radial integration data is located. Read only
fnames.save_dir  = sprintf('%s/analysis/', base_dir_work); % where to save the processed data
fnames.mcs_dir   = sprintf('%s/mcs/', base_dir_beamtime); % where to look for the mcs data, read only
fnames.pos_file  = sprintf('%s/sgalil/S%%05d.dat', base_dir_beamtime); % where to look for the positions data, read only
fnames.spec_file = sprintf('%s/', base_dir_beamtime);

% directory-name part used relative to basedir_fourier, should include an ending slash
fnames.sub_dir = [par.sample_name '/'];
% Load the data
% directory for loading integrated data, empty to choose the default
fnames.basedir_integrated_data = [ fnames.data_dir 'radial_integration/'  ]; % might need to readjust in case /S10000-19999/
fnames.basename_integrated_data = [par.user_name '_1_%05d_00000_00000_integ.mat' ];
% save Fourier components to this file, empty to choose the default
fnames.basedir_fourier = [];
fnames.filename_fourier = [];
% save figures to this directory, empty to choose the default
fnames.basedir_figures = [];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% STEP 2: perform background correction?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fnames.bgr_scan_no = [ ]; % =[] for no correction, otherwise background number, ex: 1120
fnames.bgr_point_range = [ ]; % = average over all, select range, ex: 1:201;
fnames.basename_bgr_data = [ ]; %fnames.data_dir 'analysis/radial_integration/' user_name '_1_%05d_00000_00000_integ.mat'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% STEP 3: perform transmission correction?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% transmisison correction data recorded using the MCS,
% (four_dat.f1_amp and four_dat.f2_amp will be corrected for the transmission data,
% and the transmission data will be stored on four_dat.trans_data)
% transmission data are taken from this channel (column) of the MCS data
fnames.trans_mcs_channel = 3;
fnames.basedir_trans_data = [fnames.data_dir 'online/stxm/data/']; %=[] for no correction
% first and last scan no. are set into this name
fnames.basename_trans_data = 'stxm_scans_%05d-%05d_mcs.mat';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% This section is used to see a scattering curve to determine r_sum for next step
find_rsum_values = 0; % = 1 to show integrated pattern to select ranges, = 0 to ignore
if find_rsum_values == 1
    par.test_scan = 48;       % which scan number is used to check r_sum
    par.point_range = 25;     % = [ ] to average over all points in the scan line/loopscan or = [a:b] select a range of points
    [par] = find_rsum(par, fnames);
end

%% STEP 4: define the q-ranges of interest
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define which radii ranges shall be integrated (q-resolved)
% choose the desired q-range to perform SASTT in step 8 by selecting par.which_qrange = ?;
par.r_sum = {46:61, 46:61, ...
    381:418};

fnames.save_name_suffix = {'first_range', 'first_range', ...
    'second_range' ...
     };
if (length(fnames.save_name_suffix) ~= length(par.r_sum))
    error('The number of elements in fnames.save_name_suffix is inconsistent with the number of summation ranges in par.r_sum');
end

% Background points from to, 0 0 for no background subtraction.
% with an r_sum area from 50:60 and r_bgr 5 2 the background is averaged
% from (50-5) to (50-2) and from (60+2) to (60+5)
par.r_bgr =[ 0 0; 3 1; ...
    0 0 ];
if (size(par.r_bgr,1) ~= length(par.r_sum))
    error('The number of elements in par.r_bgr is inconsistent with the number of summation ranges in par.r_sum');
end

% power law exponent, only relevant for background interpolation for
% subtraction, -1 means determination by using the intensities in the above
% specified background region, otherwise 0 to 4 as physical meaningful,
% hard-coded values
par.I_bgr_pow = [ -1, -1, ...
    -1 ...
     ];
if (length(par.I_bgr_pow) ~= length(par.r_sum))
    error('The number of elements in par.I_bgr_pow is inconsistent with the number of summation ranges in par.r_sum');
end

% if non-zero and background area specified via par.r_bgr, 
% then try peak analysis:
% 1 - simple peak analysis (try this for online analysis)
% 2 - fitting a Gaussian peak (slower than the simple analysis)
par.peakfit = [ 0, 1, ...
    0 ...
    ];
if (length(par.peakfit) ~= length(par.r_sum))
    error('The number of elements in par.peakfit is inconsistent with the number of summation ranges in par.r_sum');
end

% provide these parameters in case q is not saved in radial integration
% parameters of the setup
% par.pix_size = 0.172; %pixel size
% par.det_dist = 2.1604e3;  % detector distance
% par.lambda_A = 12.398/11.4; % 12.398/Energy (keV)

%% STEP 5: Tasks for the code and plotted output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define which steps have to be done
processing.load_data = 0; % 1 to force re-load and re-process
processing.save_fig  = 1; % 1 to save the images
processing.plot_data = 1; % 1 to avoid closing and opening the windows
processing.movie_fig = 0; % 1 to save the images as a video
processing.print_fig = 0; % 1 to print the figures (beamline printer as default)
% choose which figures to plot
fmt.fig_sym_int = 1;        % symmetric intensity over all segments: f1_amp
fmt.fig_asym_int = 1;       % amplitude of the anisotropy: f2_amp
fmt.fig_orientation = 1;    % angle of orientation (in degrees): f2_phase
fmt.fig_orient_degree = 1;  % degree of orientation from f2_amp/f1_amp
fmt.fig_orient_degree_histogram = 0; % histogram of the degree of orientation vs. orientation
fmt.fig_asym_sym_int = 1;   % combination of orientation (colorwheel), degree of orientation (hue)
fmt.fig_scatt_asym = 0;     % scattering asymmetry
fmt.fig_cos_dev = 0;        % how the fit of azimuthal integration deviates from a cosine
fmt.fig_I_pow = 0;          % slopes of I(q) in the selected q-region
% simple peak analysis
fmt.fig_simple_peak.pos = 1;       % peak position
fmt.fig_simple_peak.ampl = 1;      % peak amplitude
fmt.fig_simple_peak.width = 1;     % peak width
fmt.fig_simple_peak.all = 1;       % peak amplitude, position, width
% peak fitting
fmt.fig_fit_peak.pos = 1;       % peak position
fmt.fig_fit_peak.ampl = 1;      % peak amplitude
fmt.fig_fit_peak.width = 1;     % peak width
fmt.fig_fit_peak.all = 1;       % peak amplitude, position, width
% colormap scale: scale the caxis of each figure +/- this values times the standard
% deviation of the linear data
fmt.std_scale = 2.0;
% if the following fields are undefined or empty (i.e., [] ) then auto-scaling is
% applied to a maximum value calculated as the median value plus
% fmt.std_scale times the standard deviation of the values.
fmt.asymmetric_amplitude_scale_max = [];
fmt.symmetric_amplitude_scale_max = [];
% interpolation (smaller pixels) of the plotted data, 0 means off, greater than 1 will
% become very slow
fmt.interpol = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% STEP 6: SEGMENTATION AND SASTT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For segmentation
par.do_segmentation = 0; % = 1 to perform the segmentation
par.pixel_range = 15:1100; % range of pixels usually from 1:1225 (exclude noisy values)
par.cluster_range = 2;
% Ex: = 2 number of clusters is 2, = [2:6] to find the optimal number of
% clusters between 2 and 6 (takes about 30 sec to test 7 clusters)

% Prepare data for SASTT
par.prepare_SASTT = 0;  % =1 to prepare data to SASTT
par.which_qrange = 2;   % which q_range from par.r_sum to load for SASTT optimization
par.tomo_axis_x = 0;    % = 1 if the tomographic axis is horizontal
    % prepare data for q_resolved SASTT analysis
    perform_qres = 0; %=1 to perform q resolved, = 0 to ignore next subsection
    par.qresolved_q = [];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if perform_qres == 1 && par.prepare_SASTT == 1
        par.pixel_range = 5:650; % range of pixels usually from 5:900 (exclude noisy values)
        par.test_scan = 28;      % which scan number to visualized
        par.point_range = [];    % which point range to consider (range of points in the line): = [] for all
        par.int_integ = 40;      % how many q intervals should be considered for SASTT
        par.log_scale = 0;          % =0 for sampling q linearly and = 1 for sampling q in log scale
        plot_qres = 0;           % = 0: if not plotting is requested, = 1: if plotting is requested   
        [par] = find_qrange(par, fnames, plot_qres);
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% STEP 7: The following section should be copied from 
% spec_scanning_SAXS_caller_template_yyyy_mm_dd__hh_mm_ss.m
% This file is written by spec when the scanning_saxs macro is used
% It's saved in ~/Data10/matlab

% SAXS data
% %% relative directory identifying the sample
% fnames.rel_dir = 'sample_name/';
% % number of the first scan for this sample
% fnames.first_scan_no = 471;
% % scan dimension in intervals (not points)
% par.ind_max_x = 100;
% par.ind_max_y = 200;
% fnames.scan_no_step = 1;
% % pixel size in mm
% par.x_scale = 0.040;
% par.y_scale = 0.040;
% % the horizontal x-axis is the fast scan axis
% par.fast_axis_x = 0;
% par.snake_scan = 1;
% % perform the analysis hard coded in this sub-routine
% analyze_one_scan(fnames,par,fmt,processing);


%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
%
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing la this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS scanning SAXS package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% Additionally, any publication using the package, or any translation of the 
%     code into another computing language should cite:
%    O. Bunk, M. Bech, T. H. Jensen, R. Feidenhans'l, T. Binderup, A. Menzel 
%    and F Pfeiffer, “Multimodal x-ray scatter imaging,” New J. Phys. 11,
%    123016 (2009). (doi: 10.1088/1367-2630/11/12/123016)
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
